package linked_list;

/**
 * Trida umoznujici pouzivani oboustraneho spojoveho seznamu typu Integer.
 * Seznam je razen vzestupne.
 * 
 * @author Antonin Neumann
 * @version 1.0
 *
 */

public class Linked_list {
	
	public Node first;
	public Node last;
	public boolean verbose;
	
	public Linked_list(){
		this(true);
	}
	
	public Linked_list(boolean verbose){
		this.first = this.last = null;
		this.verbose = verbose;
	}
	
	/**
	 * Metoda pro nalezeni prvku v seznamu
	 * 
	 * @param value Integer
	 * @return Node finding node or NULL
	 */
	public Node find(int value){
		Node curr = first;
		while(curr != null){
			if(curr.value == value){
				return curr;
			}
			curr = curr.next;
		}
		return null;
	}
	
	/**
	 * Metoda prida prvek do seznamu.
	 * Prvek prida na takove misto, aby bylo zachovano vzestupne poradi.
	 * 
	 * @param value Integer
	 * @return Node current create node
	 */
	public Node add(int value){
		Node n = new Node(value);
		echo("Adding "+value);
		if(first == null){
			first = last = n;
			return n;
		}
		
		Node curr = first;
		while(curr != null){
			if(value >= last.value){ //pridej na konec
				n.prev = last;
				last.next = n;
				last = n;
				return n;
			}else if(value <= first.value){ //pridej na zacatek
				n.next = first;
				first.prev = n;
				first = n;
				return n;
			}else if(value >= curr.value && value <= curr.next.value){
				n.prev = curr;
				n.next = curr.next;
				curr.next.prev = n;
				curr.next = n;
				return n;
			}
			curr = curr.next;
		}
		return n;
	}
	
	/**
	 * Smazani prvku ze seznamu.
	 * 
	 * @param value Integer
	 * @return Boolean TRUE if node was deleted else return FALSE
	 */
	public boolean delete(int value){
		Node curr = find(value);
		if(curr != null){
			echo("Deleting "+value);
			if(curr != first)
				curr.prev.next = curr.next;
			else
				first = curr.next;
				
			if(curr != last) 
				curr.next.prev = curr.prev;
			else
				last = curr.prev;
				
			curr = null;
			return true;
		}else {
			echo("This node ("+value+") doesn't exist");
			return false;
		}
	}
	
	@Override
	public String toString() {
		String s = new String();
		if(first == null)
			s = "Any node in this list";
		
		Node curr = first;
		while(curr != null){
			s += curr.value;
			if(curr.next != null)
				s += ", ";
			curr = curr.next;
		}
		return s;
	}
	
	/**
	 * Vypise seznam na obrazovku.
	 */
	public void print(){
		System.out.println("Linked list["+this.toString()+"]");
	}
	
	/**
	 * Ukecane vypisy tzv. verbose mode
	 * @param s String message
	 */
	public void echo(String s){
		if(this.verbose)
			System.out.println(s);
	}
}

/**
 * Trida reprezentujici jeden prvek seznamu
 * 
 * @author Antonin Neumann
 * @version 1.0
 */
class Node {
	/* atributy jsou public, pro jednodussi manipulaci se seznamem */
	public int value;
	public Node next;
	public Node prev;
	
	Node (int value){
		this.value = value;
		this.next = null;
		this.prev = null;
	}
}